/*******************************************************************************
 * Copyright (C) 2002, 2003
 * ingenieurbuero fuer innovative informationstechnik (iiit)
 * Dipl.-Ing. Joerg Beckmann, Dortmund, Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * version $Id: CacheManager.java,v 1.26 2003/04/14 20:39:56 joerg Exp $
 ******************************************************************************/

package de.iiit.access.server.plugins.cachemanager;

import de.iiit.access.server.*;
import de.iiit.access.server.api.*;
import de.iiit.access.server.util.*;

import de.iiit.xmlconfig.*;
import de.iiit.ldap.*;
import de.iiit.jdbc.*;

import java.io.*;
import java.nio.channels.*;
import java.util.*;
import java.util.regex.*;
import javax.naming.*;

import org.apache.log4j.Logger;

/** The CacheManager manages the 2<sup>nd</sup>-level cache stored in databases.
 * To do this, it reads and parses the OpenLDAP server's replication log file.
 * If a fatal error occurs, the whole AccessServer will be aborted by a call to
 * System.exit().
 */
public class CacheManager implements ThreadPluginIf
{
    /** CVS Version Tag */
    private static final String vcid = "$Id: CacheManager.java,v 1.26 2003/04/14 20:39:56 joerg Exp $";
    
    private static final String REPLICATION_FILE   = "ReplicationFile";
    private static final String REFRESH_INTERVAL   = "RefreshInterval";
    
    private static final String MD5PATTERN_LENGTH  = "Md5PatternLength";

    private static final String SMTP_SERVER        = "SMTPServer";
    private static final String SMTP_PORT          = "SMTPPort";
    private static final String SMTP_USER          = "SMTPUser";
    private static final String SMTP_PASSWORD      = "SMTPPassword";
    private static final String SMTP_MAIL_TO       = "SMTPMailTo";
    
    private static final String JDBC_DRIVER        = "ClassName";
    private static final String JDBC_URL           = "Url";
    private static final String JDBC_USERNAME      = "UserName";
    private static final String JDBC_PASSWORD      = "Password";
    private static final String JDBC_CONNECTIONS   = "Connections";
    
    private static final String JDBC_DRIVER_CONFIG = "JdbcDriver";
    
    private static final String ADMIN_DATABASE_CONFIG = "AdminDatabase";
    private static final String CACHE_DATABASE_CONFIG = "CacheDatabase";

    private String replicationFile  = null;
    private int    refreshIntervall = 10000;
    
    private String smtpServer       = null;
    private int    smtpPort         = -1;
    private String smtpUser         = null;
    private String smtpPassword     = null;
    private String smtpMailTo       = null;

    private CacheToDoHandler         toDoHandler  = null;
    private CacheManagerErrorHandler errorHandler = null;
    
    private Configuration config                = null;
    private Configuration adminDbConfig         = null;
    private Vector        cacheDbConfigVector   = null;
    private Vector        jdbcConfigVector      = null;

    private JdbcConnectionPool adminPool = null;
    private Vector cachePools = new Vector();
    private Hashtable md5PatternIndex = new Hashtable();
    private int md5PatternLength = 0;

    private Set userList = null;
    
    private Logger logger = Logger.getLogger(this.getClass());

    private LdapUtil ldapUtil = null;
    
    private static final char[] hexadecimal = 
            {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 
             'a', 'b', 'c', 'd', 'e', 'f'};
             
    /** Creates a new instance of CacheDatabaseManager */
    public CacheManager()
    {
    }
    
    /** Initializes the CacheManagerPlugin
     * @param config the configuration of the cache manager
     */    
    public void initialize(Configuration config)
    {
        boolean cfgError = false;
        Configuration ldapConfig = null;
        
        this.config = config;
        
        if (config == null)
        {
            logger.fatal("No CacheManagerPlugin configuration found");
            cfgError = true;
        }
        else
            ldapConfig = config.getSubConfiguration("LdapConfig");
        
        if (ldapConfig == null)
            ldapConfig = AccessServer.getSubConfiguration("LdapConfig");
        
        if (ldapConfig == null)
        {
            logger.fatal("The CacheManagerPlugin can not work without a LDAP configuration");
            cfgError = true;
        }
        else
        {
            ldapUtil = new LdapUtil();
            ldapUtil.initialize(ldapConfig);
        }
                
        Vector v = config.getSubConfigurations(ADMIN_DATABASE_CONFIG);
 
        if (v != null)
            adminDbConfig = (Configuration) v.get(0);
        else
        {
            logger.fatal("No configuration for admin db found");
            cfgError = true;
        }
        
        cacheDbConfigVector = config.getSubConfigurations(CACHE_DATABASE_CONFIG);
        
        if (cacheDbConfigVector == null)
        {
            logger.fatal("No configuration for cache db found");
            cfgError = true;
        }
        
        replicationFile  = config.getAttribute(REPLICATION_FILE);    
        refreshIntervall = Integer.parseInt(config.getAttribute(REFRESH_INTERVAL, "10")) * 1000;

        md5PatternLength = Integer.parseInt(config.getAttribute(MD5PATTERN_LENGTH, "0"));
        
        smtpServer       = config.getAttribute(SMTP_SERVER);
        smtpPort         = Integer.parseInt(config.getAttribute(SMTP_PORT, "-1"));
        smtpUser         = config.getAttribute(SMTP_USER);
        smtpPassword     = config.getAttribute(SMTP_PASSWORD);
        smtpMailTo       = config.getAttribute(SMTP_MAIL_TO);

        if (smtpServer == null || smtpMailTo == null)
        {
            logger.warn("No SMTP configuration found");
        }
        
        if ((replicationFile == null) || (replicationFile.equals("")))
        {
            logger.fatal("No LDAP replication file configured");
            cfgError = true;
        }
        
        jdbcConfigVector = config.getSubConfigurations(JDBC_DRIVER_CONFIG);

        if (jdbcConfigVector == null)
        {
            logger.fatal("No JDBC configuration found");  
            cfgError = true;
        }
        
        if (cfgError)
            System.exit(1);

        try
        {
            JdbcUtil.registerJdbcDriver(jdbcConfigVector);
        }
        catch(Exception e)
        {
            logger.fatal("Error while registering JDBC driver", e);  
            System.exit(1);
        }
    
        initializeAdminDb();
        initializeCacheDb();
    }

    /** Starts the CacheManager's background threads.
     * This method is called by the AccessServer after the initialization of all sub-moduls.
     */    
    public void start()
    {               
        errorHandler = new CacheManagerErrorHandler(this);

        toDoHandler = new CacheToDoHandler(this);
        toDoHandler.start();
    }

    /** This method is called by the AccessServer when the background threads should
     * stop because of a shutdown of the AccessServer itself.
     */
    public void shutdown()
    {
        if (toDoHandler != null)
            toDoHandler.shutdown();
    }
    
    private void initializeAdminDb()
    {
        String jdbcUrl         = adminDbConfig.getAttribute(JDBC_URL);
        String jdbcUsername    = adminDbConfig.getAttribute(JDBC_USERNAME);
        String jdbcPassword    = adminDbConfig.getAttribute(JDBC_PASSWORD);
        int    jdbcConnections = adminDbConfig.getIntAttribute(JDBC_CONNECTIONS);
            
        adminPool = new JdbcConnectionPool(jdbcUrl, jdbcUsername, jdbcPassword, jdbcConnections);
    }
    
    private void initializeCacheDb()
    {
        JdbcConnectionPool pool = null;
        
        int pattern = 1 << md5PatternLength;
        
        int iMax = cacheDbConfigVector.size();
        
        if (iMax < pattern)
        {
            logger.fatal("Not enough cache databases configured for md5PatternLength =  <" + md5PatternLength + ">.");
            System.exit(1);
        }

        for (int i = 0; i < iMax; i++)
        {
            Configuration cfg = (Configuration) cacheDbConfigVector.get(i);
            
            String jdbcUrl         = cfg.getAttribute(JDBC_URL);
            String jdbcUsername    = cfg.getAttribute(JDBC_USERNAME);
            String jdbcPassword    = cfg.getAttribute(JDBC_PASSWORD);
            int    jdbcConnections = cfg.getIntAttribute(JDBC_CONNECTIONS);

            pool = new JdbcConnectionPool(jdbcUrl, jdbcUsername, jdbcPassword, jdbcConnections);
                                
            cachePools.add(i, pool);
        }

        for (int i = 0; i < 256; i++)
        {
            int low  = i & 0x0f;
            int high = (i & 0xf0) >> 4;
            String key = "" + hexadecimal[high] + hexadecimal[low];
            Integer value = new Integer(i % pattern);
            
            md5PatternIndex.put(key, value);
        }
    }     

    /** Retrieves an instance of the CacheMnagerErrorHandler
     * @return an instance of the CacheMnagerErrorHandler
     */    
    protected CacheManagerErrorHandler errorHandler()
    {
        return errorHandler;
    }
    
    /** Retrieves a handle to the admin database
     * @return the handle to the admin database
     */    
    protected JdbcConnectionPool getAdminPool() // throws SQLException
    {
        return adminPool;
    }
    
    /** Retrieves the number of configured cache databases.
     * @return the number of configured cache databases
     */    
    protected int getCachePoolCount()
    {
        return cacheDbConfigVector.size();
    }
    
    /** Retrieves one of the configured cache database handles
     * @param dbnum The number of the database handle
     * @return the requested database handle
     */    
    protected JdbcConnectionPool getCacheConnectionPool(int dbnum)
    {
        return (JdbcConnectionPool) cachePools.get(dbnum);
    }
    
    /** Retrieves the handle of cache database responsible for a distinct MD5 checksum
     * @param md5sum The MD5 sum for wich the database handle is needed
     * @return the requested database handle
     */    
    protected JdbcConnectionPool getCacheConnectionPool(String md5sum)
    {
        String key = md5sum.substring(md5sum.length() - 2);
        int dbnum = ((Integer) md5PatternIndex.get(key)).intValue();

        return getCacheConnectionPool(dbnum);
    }
    
    /** Retrieves the configured name of the LDAP replication file.
     * @return the name of the LDAP replication file
     */    
    protected String replicationFile()
    {
        return replicationFile;
    }
    
    /** Retrieves the configured intervall to read the LDAP replication file
     * @return the configured intervall
     */    
    protected int refreshIntervall()
    {
        return refreshIntervall;
    }

    /** Retrieves the configured name of the SMTP server to use for sending error mails.
     * @return the name of the SMTP server
     */    
    protected String smtpServer()
    {
        return smtpServer;
    }
    
    /** Retrieves the configured port number of the SMTP server
     * @return the port number
     */    
    protected int smtpPort()
    {
        return smtpPort;
    }

    /** Retrieves the configured user name to use for SMTP connections
     * @return the user name
     */    
    protected String smtpUser()
    {
        return smtpUser;
    }
    
    /** Retrieves the configured password to use for SMTP connections
     * @return the password
     */    
    protected String smtpPassword()
    {
        return smtpPassword;
    }
    
    /** Retrieves the configured address to use for error mails.
     * @return the mail address
     */    
    protected String smtpMailTo()
    {
        return smtpMailTo;
    }
    
    /** Retrieves a ready-to-use LdapUtil object.
     * @return the LdapUtil object to use for LDAP requests
     */    
    protected LdapUtil getLdapUtil()
    {
        return ldapUtil;
    }

}

/**
 * $Log: CacheManager.java,v $
 * Revision 1.26  2003/04/14 20:39:56  joerg
 * Initializing JdbcConnectionPools with pre-opened connections.
 *
 * Revision 1.25  2003/04/13 20:28:01  joerg
 * Package structure modified
 *
 * Revision 1.24  2003/04/13 20:16:41  joerg
 * Package structure modified
 *
 * Revision 1.23  2003/01/01 21:04:17  joerg
 * Copyright-Statement aktualisiert
 *
 * Revision 1.22  2002/12/30 22:26:37  joerg
 * RefreshTime jetzt in Sekunden statt Millisekunden.
 *
 * Revision 1.21  2002/12/27 20:20:42  joerg
 * JavaDoc aktualisiert.
 * Fehlermeldungen verbessert.
 *
 * Revision 1.20  2002/12/24 21:04:33  joerg
 * Umbau der Paketstruktur
 * iiitLdapPlugin integriert
 * JavaDoc-Kommentare weiter vervollstaendigt.
 *
 * Revision 1.19  2002/12/23 11:25:52  joerg
 * shutdown()-Methode hinzugefuegt.
 *
 * Revision 1.18  2002/12/21 19:55:03  joerg
 * Nicht mehr benoetigte Methoden entfernt, interne Methoden auf
 * private oder protected geaendert.
 * JavaDoc Kommentare ergaenzt.
 *
 * Revision 1.17  2002/12/19 15:54:33  joerg
 * Paket umbenannt in iiitLdapPlugin
 *
 * Revision 1.16  2002/12/09 21:15:51  joerg
 * AccessServerThreadPluginIf umbenannt in ThreadPluginIf
 *
 * Revision 1.15  2002/12/08 19:34:11  joerg
 * Auskommentierten Code entfernt
 *
 * Revision 1.14  2002/12/08 16:37:33  joerg
 * Aufraeumungsarbeiten nach dem grossen Umbau
 *
 * Revision 1.13  2002/12/08 16:09:46  joerg
 * Paket-Struktur ueberarbeitet
 *
 * Revision 1.12  2002/12/08 14:35:17  joerg
 * Plugin-Interface neu strukturiert
 *
 * Revision 1.11  2002/12/08 14:17:02  joerg
 * Fehler von der letzten Aenderung behoben
 *
 * Revision 1.10  2002/12/08 13:54:46  joerg
 * Das LdapPlugin wird jetzt hier direkt geladen und nicht mehr
 * ueber den AccessServer
 *
 * Revision 1.9  2002/11/27 22:32:23  joerg
 * Aufteilung der Cache-Daten auf verschiedene Datenbanken
 *
 * Revision 1.8  2002/11/27 12:13:49  joerg
 * Beim Erkennen fehlerhafte Ausdruecke kann jetzt automatisch
 * eine Fehler-Mail verschickt werden.
 *
 * Revision 1.7  2002/11/26 21:54:54  joerg
 * Interne Vereinfachungen
 *
 * Revision 1.6  2002/11/26 10:56:40  joerg
 * Package exprparser durch parser erstzt.
 *
 * Revision 1.5  2002/11/21 21:49:45  joerg
 * Umstellung auf JdbcConnectionPool
 *
 * Revision 1.4  2002/11/21 08:37:03  joerg
 * Auskommentierten Code beseitigt
 *
 * Revision 1.3  2002/11/20 20:41:29  joerg
 * Klassen fuer das HAndling der DB-Tabellen in Packages
 * de.iiit.AccessServer.db.* verschoben
 *
 * Revision 1.2  2002/11/20 20:27:17  joerg
 * Bezuege auf UserList entfernt
 * registerJdbcDriver nach de.iiit.util.JdbcUtil ausgelagert
 *
 * Revision 1.1  2002/11/18 22:07:46  joerg
 * Klasse CacheManager umbenannt in CacheManagerPlugin
 *
 * Revision 1.2  2002/11/18 21:21:52  joerg
 * LDAP-Anbindung auf LdapPlugin umgestellt.
 *
 * Revision 1.1  2002/11/18 10:17:49  joerg
 * Klassen des CacheManagers in eigenes Package verschoben
 *
 * Revision 1.1  2002/11/17 22:03:26  joerg
 * Neue Klassen fuer 2nd-level Cache
 *
 */
